@@ -19,6 +19,7 @@
 
 #include <bonobo-conf/bonobo-config-bag.h>
 #include <bonobo-conf/bonobo-config-property.h>
+#include <bonobo-conf/bonobo-config-subdir.h>
 
 #include <bonobo-conf/bonobo-property-editor.h>
 #include <bonobo-conf/bonobo-property-bag-editor.h>
@@ -69,8 +70,8 @@ config_resolve (BonoboMoniker               *moniker,
 		CORBA_Environment           *ev)
 {
 	Bonobo_Moniker         parent;
-	Bonobo_ConfigDatabase  db;
-	const gchar           *name;
+	Bonobo_ConfigDatabase  db, dbproxy;
+	const gchar           *name, *pdn;
 
 	parent = bonobo_moniker_get_parent (moniker, ev);
 	if (BONOBO_EX (ev)) 
@@ -78,6 +79,12 @@ config_resolve (BonoboMoniker               *moniker,
 
 	if (parent != CORBA_OBJECT_NIL) {
 
+		pdn = Bonobo_Moniker_getDisplayName (parent, ev);
+		if (BONOBO_EX (ev) || pdn == NULL) {
+			bonobo_object_release_unref (parent, NULL);
+			return CORBA_OBJECT_NIL;
+		}
+		
 		db = Bonobo_Moniker_resolve (parent, options, 
 		        "IDL:Bonobo/ConfigDatabase:1.0", ev);
 
@@ -87,8 +94,9 @@ config_resolve (BonoboMoniker               *moniker,
 			return CORBA_OBJECT_NIL;
 	} else {
 
-		db = bonobo_get_object (DEFAULTDB, "Bonobo/ConfigDatabase", 
-					ev);
+		pdn = DEFAULTDB;
+
+		db = bonobo_get_object (pdn, "Bonobo/ConfigDatabase", ev);
 
 		if (BONOBO_EX (ev) || db == CORBA_OBJECT_NIL)
 			return CORBA_OBJECT_NIL;
@@ -96,15 +104,35 @@ config_resolve (BonoboMoniker               *moniker,
 
 	name = bonobo_moniker_get_name (moniker);
 
-	if (!strcmp (requested_interface, "IDL:Bonobo/ConfigDatabase:1.0"))
-		return db;
+	if (!strcmp (requested_interface, "IDL:Bonobo/ConfigDatabase:1.0")) {
+	
+		dbproxy = db;
+		// dbproxy =  bonobo_config_proxy_new (db, name, pdn);
+		// bonobo_object_release_unref (db, NULL);
+
+		if (dbproxy != CORBA_OBJECT_NIL)
+			return dbproxy;
+
+		bonobo_exception_set (ev, ex_Bonobo_Moniker_InterfaceNotFound);
+
+		return CORBA_OBJECT_NIL;		
+	}
 
 	if (!strcmp (requested_interface, "IDL:Bonobo/PropertyBag:1.0")) {
 		BonoboConfigBag *bag;
 	
-		bag = bonobo_config_bag_new (db, name);
+		dbproxy = db;
+		//dbproxy =  bonobo_config_proxy_new (db, NULL, pdn);
+		//bonobo_object_release_unref (db, NULL);
 
-		bonobo_object_release_unref (db, NULL);
+		if (dbproxy == CORBA_OBJECT_NIL) {
+			bonobo_exception_set (ev, 
+			        ex_Bonobo_Moniker_InterfaceNotFound);
+			return CORBA_OBJECT_NIL;
+		}
+
+		bag = bonobo_config_bag_new (dbproxy, name);
+		bonobo_object_release_unref (dbproxy, NULL);
 	
 		if (bag)
 			return (Bonobo_Unknown) CORBA_Object_duplicate (
@@ -120,7 +148,18 @@ config_resolve (BonoboMoniker               *moniker,
  	if (!strcmp (requested_interface, "IDL:Bonobo/Property:1.0")) {
 		BonoboConfigProperty *prop;
 
-		prop = bonobo_config_property_new (db, name);
+		dbproxy = db;
+		//dbproxy =  bonobo_config_proxy_new (db, NULL, pdn);
+		//bonobo_object_release_unref (db, NULL);
+
+		if (dbproxy == CORBA_OBJECT_NIL) {
+			bonobo_exception_set (ev, 
+			        ex_Bonobo_Moniker_InterfaceNotFound);
+			return CORBA_OBJECT_NIL;
+		}
+
+		prop = bonobo_config_property_new (dbproxy, name);
+		bonobo_object_release_unref (dbproxy, NULL);
 
 		if (prop)
 			return (Bonobo_Unknown) CORBA_Object_duplicate (
@@ -134,7 +173,23 @@ config_resolve (BonoboMoniker               *moniker,
  	if (!strcmp (requested_interface, "IDL:Bonobo/Control:1.0")) {
 		if (Bonobo_ConfigDatabase_dirExists (db, name, ev) &&
 		    !BONOBO_EX (ev)) {
-			return create_bag_editor (db, name, ev);
+			Bonobo_Unknown o;
+
+			dbproxy = db;
+			//dbproxy =  bonobo_config_proxy_new (db, NULL, pdn);
+			//bonobo_object_release_unref (db, NULL);
+
+			if (dbproxy == CORBA_OBJECT_NIL) {
+				bonobo_exception_set (ev, 
+				        ex_Bonobo_Moniker_InterfaceNotFound);
+				return CORBA_OBJECT_NIL;
+			}
+
+			o = create_bag_editor (dbproxy, name, ev);
+
+			bonobo_object_release_unref (dbproxy, NULL);
+
+			return o;
 		}
 	} 
 		
